Imports System.Drawing.Drawing2D
Imports System.Collections.Generic

Public Class DrawObject : Implements IDisposable

    Private _TemplatePath As New Drawing2D.GraphicsPath
    Private _pthLine As New Drawing2D.GraphicsPath
    Private _pthWidenedLine As New GraphicsPath
    Private _Pen As New Pen(Color.Red, 1)
    Private _Matrix As New Drawing2D.Matrix
    Private _Bounds As Rectangle
    Private _Control As Control

    Private _Rotation As Double = 0.0#
    Private _Location As New Point(100, 100)
    Private _Scale As New Size(100, 100)

    ' Bei aktivierter Kantenglttung liegen einzelne Punkte auerhalb der exakt berechneten Figur
    ' ApplyChanges() bercksichtigt dieses, um Zeichnungs-Fehler zu vermeiden
    Private _IsSmoothing As Boolean = True
    Private _SmoothModes As IList(Of SmoothingMode) = New SmoothingMode() { _
       SmoothingMode.AntiAlias, SmoothingMode.HighQuality}

    Public Sub New(ByVal Control As Control)
        _Control = Control
    End Sub

    ''' <summary> Offenlegung des Stiftes, mit dem gezeichnet wird </summary>
    Public ReadOnly Property Pen() As Pen
        Get
            Return _Pen
        End Get
    End Property

    ''' <summary>
    ''' Zur Aufnahme des Musters der Figur. Verwenden Sie Grenangaben im Bereich von +-1.0, 
    ''' und legen Sie mit Public Property Scale() die Gre der Darstellung fest.
    ''' </summary>
    Public ReadOnly Property TemplatePath() As GraphicsPath
        Get
            Return _TemplatePath
        End Get
    End Property

    ''' <summary> Vergrerungs-Faktor in X / Y - Richtung </summary>
    Public Property Scale() As Size
        Get
            Return _Scale
        End Get
        Set(ByVal NewValue As Size)
            _Scale = NewValue
        End Set
    End Property

    ''' <summary> Winkel zur X-Achse in Degree (360) </summary>
    Public Property Rotation() As Double
        Get
            Return _Rotation
        End Get
        Set(ByVal NewValue As Double)
            _Rotation = NewValue Mod 360
        End Set
    End Property

    ''' <summary> Position des Nullpunktes </summary>
    Public Property Location() As Point
        Get
            Return _Location
        End Get
        Set(ByVal NewValue As Point)
            _Location = NewValue
        End Set
    End Property

    '''<summary>
    ''' wendet die aktuellen Einstellungen auf die Figur an, stt Neuzeichnen an
    ''' </summary>
    Public Sub ApplyChanges()
        ' tckisch: wird bei Matrix-Befehlen die jeweilige berladung ohne Parameter "Order As
        ' MatrixOrder" gewhlt, erhlt man i.A. unerwartete Ergebnisse, denn die Voreinstellung
        ' ist "MatrixOrder.Prepend"
        _Matrix.Reset()
        _Matrix.Scale(_Scale.Width, _Scale.Height, MatrixOrder.Append)
        _Matrix.Translate(_Location.X, _Location.Y, MatrixOrder.Append)
        _Matrix.RotateAt(CSng(_Rotation), _Location, MatrixOrder.Append)
        _pthLine.Reset()
        _pthLine.AddPath(_TemplatePath, False)
        _pthLine.Transform(_Matrix)
        ' Invalidate-Rectangle ermitteln, dabei Stift und ggfs. Kantenglttung bercksichtigen
        ' Die berladungen von GraphicsPath.GetBounds(), die den Stift bercksichtigen
        ' sollen, sind buggy, und liefern viel zu groe Bounds. Einzig GetBounds() fr
        ' auszufllende Pfade arbeitet korrekt.
        ' Das bedeutet, da Graphics.DrawPath(Path, Pen) nicht benutzt werden kann, sondern nur
        ' Graphics.FillPath(Path).
        ' Zum Glck generiert GraphicsPath.Widen(Pen) genau die Umrilinie der
        ' **Flche des Striches**, mit dem Pen die Linie zeichnen wrde. Diesen mit Pen geweiteten
        ' Pfad auszufllen ist also quivalent dazu, den eigentlichen Pfad mit Pen zu zeichnen.
        ' Beachte: fr Zeichnungen ausgefllter Figuren ist dieser Workaround unntig.
        _pthWidenedLine.Reset()
        _pthWidenedLine.AddPath(_pthLine, False)
        _pthWidenedLine.Widen(_Pen)
        Dim NewBounds As RectangleF = _pthWidenedLine.GetBounds
        If _IsSmoothing Then NewBounds.Inflate(0.5F, 0.5F)
        ' Invalidate(Rectangle) wird fr zwei verschiedene Bereiche aufgerufen:
        ' 1) bisheriger Zeichnungsbereich der Figur (Lschen)
        ' 2) neuer Zeichnungsbereich der Figur
        ' Invalidate() veranlasst die CLR, kurze Zeit darauf ein  Paint-Event auszulsen (nur
        ' eines!), mit einem Clip-Bereich, welcher **alle** invalidierten Bereiche vereint.
        ' Dieses erhellt, wie extrem der ganze Vorgang darauf optimiert, das zeitkritische
        ' Zeichnen so selten wie mglich auszufhren, und auch nur die minimal erforderliche
        ' Flche zu "bemalen".
        _Control.Invalidate(_Bounds)
        _Bounds = Rectangle.Ceiling(NewBounds)
        _Control.Invalidate(_Bounds)
    End Sub

    Public ReadOnly Property Contains(ByVal Pt As PointF) As Boolean
        Get
            Return _pthLine.IsVisible(Pt)
        End Get
    End Property

    ''' <summary> zeichnet die Figur </summary>
    ''' <remarks>
    ''' wird letztlich von der CLR aufgerufen, **nicht** in direkter Reaktion auf Benutzereingaben
    ''' </remarks>
    Public Sub Draw(ByVal G As Graphics)
        _IsSmoothing = _SmoothModes.Contains(G.SmoothingMode)
        G.FillPath(_Pen.Brush, _pthWidenedLine)
    End Sub

#Region " IDisposable Support "

    ''' <summary>
    ''' Ressourcen-Freigabe: fr alle Objekte, die IDisposable implementieren, Dispose() aufrufen
    ''' </summary>
    Protected Overridable Sub Dispose(ByVal disposing As Boolean)
        If Not disposing Then Return
        Static IsDisposed As Boolean = False
        If IsDisposed Then Return
        IsDisposed = True
        For Each D As IDisposable In New IDisposable() { _
                _TemplatePath, _pthLine, _pthWidenedLine, _Matrix, _Pen}
            D.Dispose()
        Next
    End Sub

    Public Sub Dispose() Implements IDisposable.Dispose
        Dispose(True)
        GC.SuppressFinalize(Me)
    End Sub

#End Region 'IDisposable Support

End Class
