Imports System.Drawing.Drawing2D

''' <summary> 
''' Auf diesem Form wird eine verschiebbare Pfeil-Figur gezeichnet, die sich
''' zustzlich auf die Mausposition ausrichtet
''' </summary>
Public Class frmDrawObject

    Private _Arrow As New DrawObject(Me)

    ''' <summary>Differenz zw. Nullpunkt und Drag-Anfasspunkt</summary>
    ''' <remarks>
    ''' die Nullable-Struktur tranportiert zustzlich die Information, 
    ''' ob _GrabOffset berhaupt gesetzt ist
    ''' </remarks>
    Private _GrabOffset As Nullable(Of Size)

    Public Sub New()
        InitializeComponent()
        'Pfeil-Figur aufsetzen (mit allem Drum und Dran)
        _Arrow.Pen.Width = 10
        _Arrow.Pen.LineJoin = LineJoin.Round
        _Arrow.TemplatePath.AddLines( _
              New PointF() { _
                 New PointF(0, -0.2), _
                 New PointF(0.6, -0.2), _
                 New PointF(0.6, -0.5), _
                 New PointF(1, 0), _
                 New PointF(0.6, 0.5), _
                 New PointF(0.6, 0.2), _
                 New PointF(0, 0.2)})
        _Arrow.TemplatePath.AddArc(New RectangleF(-0.2, -0.2, 0.4, 0.4), 90, 180)
        _Arrow.Location = New Point(250, 200)
        _Arrow.Scale = New Size(100, 100)
        _Arrow.ApplyChanges()
    End Sub


    ''' <summary>Draggen des _Arrows starten</summary>
    Protected Overrides Sub OnMouseDown(ByVal e As MouseEventArgs)
        MyBase.OnMouseDown(e)
        If _Arrow.Contains(e.Location) Then _
            _GrabOffset = New Size(e.Location - New Size(_Arrow.Location))
    End Sub

    ''' <summary> Figur entweder draggen, oder auf die Maus ausrichten </summary>
    Protected Overrides Sub OnMouseMove(ByVal e As MouseEventArgs)
        MyBase.OnMouseMove(e)
        With _Arrow
            If _GrabOffset.HasValue Then
                .Location = e.Location - _GrabOffset.Value
            Else
                .Rotation = GetAngle(.Location, e.Location) * 180 / Math.PI 'in Degree!
            End If
            .ApplyChanges()
        End With
    End Sub

    ''' <summary>Draggen beenden</summary>
    Protected Overrides Sub OnMouseUp(ByVal e As MouseEventArgs)
        MyBase.OnMouseUp(e)
        _GrabOffset = Nothing
    End Sub

    Protected Overrides Sub OnPaint(ByVal e As PaintEventArgs)
        MyBase.OnPaint(e)
        If ckShowClip.Checked Then
            ' ggfs. ClipBereich visualisieren
            ' e.Graphics.Clear() frbt nur den Clipbereich neu
            e.Graphics.Clear(ColorGenerator.GetNextColor)
        End If
        e.Graphics.SmoothingMode = SmoothingMode.HighQuality
        _Arrow.Draw(e.Graphics)
        If ckAutoRun.Checked Then
            'Ereigniskette: ApplyChanges() lst das nchste Paint-Ereignis aus
            'So zeigt sich die maximale Zeichnungs-Geschwindigkeit
            With _Arrow
                .Rotation += 2
                .ApplyChanges()
            End With
        End If
    End Sub

    ''' <summary>Ressourcen-Freigabe</summary>
    Protected Overrides Sub Dispose(ByVal disposing As Boolean)
        'IMHO gehrt der Dispose-Override nicht in den Designer-Code (wurde da herausgeholt)
        If disposing Then
            If _Arrow IsNot Nothing Then
                _Arrow.Dispose()
                _Arrow = Nothing
            End If
            If components IsNot Nothing Then components.Dispose()
        End If
        MyBase.Dispose(disposing)
    End Sub

    Private Sub ckAutoRun_CheckedChanged(ByVal sender As Object, ByVal e As EventArgs) _
          Handles ckAutoRun.CheckedChanged
        If ckAutoRun.Checked Then Me.Invalidate()
    End Sub

    Private Sub ckDoubleBuffer_CheckedChanged(ByVal sender As Object, ByVal e As EventArgs) _
          Handles ckDoubleBuffer.CheckedChanged
        'reduziert flackern, indem der Zeichenvorgang in einen Puffer umgeleitet wird.
        'Die Performance-Kosten machen sich erst bemerkbar, wenn extrem schnelle
        'Bewegungen darzustellen sind.
        'Beachte: DoubleBuffered ist Protected. Beim Malen auf fremde Controls also kein Zugriff.
        'Zusatz-Info DoubleBuffered-unvernderliche Einstellungen: Picturebox True, Panel: False
        MyBase.DoubleBuffered = ckDoubleBuffer.Checked
    End Sub

    ''' <summary>berechnet den Winkel eines Vektors zur X-Achse</summary>
    Private Function GetAngle(ByVal ptFrom As PointF, ByVal ptTo As PointF) As Double
        ptTo -= New SizeF(ptFrom) 'ptTo in Relation zu ptFrom setzen
        With ptTo
            Dim Amount As Double = Math.Sqrt(.X ^ 2 + .Y ^ 2)
            If Amount > 0 Then           'Nulldivision vermeiden
                If .X > 0 Then
                    Return Math.Asin(.Y / Amount)
                Else
                    Return Math.PI - Math.Asin(.Y / Amount)
                End If
            End If
        End With
    End Function

End Class
